local mod = ...
core = {}
apse = {}
narthex = {}
tower = {}
roof = {}


----------------------
-- Asset Processors --
----------------------
mod:registerAssetProcessor("models/nave.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/crossing.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/apse.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/narthex.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/portal.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/portal_m.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/towers.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })
mod:registerAssetProcessor("models/stairs.fbx", {  	DataType = "BUILDING_ASSET_PROCESSOR" })

---------------------
-- Register materials
---------------------
mod:registerAssetId("models/nave.fbx/Materials/Limestone", "PREFAB_CATHEDRAL_LIMESTONE_MATERIAL")
mod:registerAssetId("models/crossing.fbx/Materials/Limestone", "PREFAB_CATHEDRAL_LIMESTONE_MATERIAL")
mod:registerAssetId("models/apse.fbx/Materials/Limestone", "PREFAB_CATHEDRAL_LIMESTONE_MATERIAL")
mod:registerAssetId("models/narthex.fbx/Materials/Limestone", "PREFAB_CATHEDRAL_LIMESTONE_MATERIAL")


-------------------------------------
-- Function to register building part
-------------------------------------
function registerBuildingPart(_array)
    _nodePrefixLower = _array[1]
    _nodePrefixUpper = string.upper(_nodePrefixLower)
    _isScalable = _array[2]
    _fbxPath = _array[3]
    _minScale = _array[4]
    _tableMenu = _array[5]
    _capacity = _array[6]
    _isPlatform = _array[7]
    _polygon = _array[8]
    _nodeType = _array[9]
    if _isScalable then
      _root = "__Root"
  	  _bottom = "_Bottom"
  	  mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "__Root_Part",  "PREFAB_" .. _nodePrefixUpper .. "_ROOT")
        
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Top_Part",     "PREFAB_" .. _nodePrefixUpper .. "_TOP")
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Middle_Part",  "PREFAB_" .. _nodePrefixUpper .. "_MIDDLE")
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Bottom_Part",  "PREFAB_" .. _nodePrefixUpper .. "_BOTTOM")
      
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Top_Construction",     "PREFAB_" .. _nodePrefixUpper .. "_TOP_CONSTR")
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Middle_Construction",  "PREFAB_" .. _nodePrefixUpper .. "_MIDDLE_CONSTR")
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Bottom_Construction",  "PREFAB_" .. _nodePrefixUpper .. "_BOTTOM_CONSTR")
      
      mod:register({
          DataType = "BUILDING_PART",
          Id = _nodePrefixUpper .. "_PART",
          Name = _nodePrefixUpper .. "_PART_NAME",
          Description = _nodePrefixUpper .. "_PART_DESC",
          
          ConstructorData = {
              DataType = "BUILDING_CONSTRUCTOR_SCALER",
              CoreObjectPrefab = "PREFAB_" .. _nodePrefixUpper .. "_ROOT",
              EndPart = _nodePrefixUpper .. "_TOP",
              FillerList = {
                  _nodePrefixUpper .. "_MIDDLE"
              },
              BasementFillerList = {
                  _nodePrefixUpper .. "_BOTTOM"
              },
              MinimumScale = _minScale,
              IsVertical = true
          },
          BuildingZone = {
              ZoneEntryList = {
        			{
        				Polygon = polygon.createRectangle( _polygon ),
        				Type = {
        					DEFAULT = true,
        					NAVIGABLE = true
        				}
        			}
      		  }
    	  },
          
      })
      mod:register({
      	DataType = "BUILDING_PART",
      	Id = _nodePrefixUpper .. "_TOP",
      	ConstructorData = {
      		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
      		CoreObjectPrefab = "PREFAB_" .. _nodePrefixUpper .. "_TOP"
      	},
        ConstructionVisual = "PREFAB_" .. _nodePrefixUpper .. "_TOP_CONSTR",
        Cost = {
            RessourcesNeeded = {
                { Resource = "WOOD", Quantity = 1 },
                { Resource = "STONE", Quantity = 1 },
		    }
	    },
      })
      mod:register({
      	DataType = "BUILDING_PART",
      	Id = _nodePrefixUpper .. "_MIDDLE",
      	ConstructorData = {
      		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
      		CoreObjectPrefab = "PREFAB_" .. _nodePrefixUpper .. "_MIDDLE"
      	},
        ConstructionVisual = "PREFAB_" .. _nodePrefixUpper .. "_MIDDLE_CONSTR",
        Cost = {
            RessourcesNeeded = {
                { Resource = "WOOD", Quantity = 1 },
                { Resource = "STONE", Quantity = 1 },
		    }
	    },
      })
      mod:register({
      	DataType = "BUILDING_PART",
      	Id = _nodePrefixUpper .. "_BOTTOM",
      	ConstructorData = {
      		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
      		CoreObjectPrefab = "PREFAB_" .. _nodePrefixUpper .. "_BOTTOM"
      	},
        ConstructionVisual = "PREFAB_" .. _nodePrefixUpper .. "_BOTTOM_CONSTR",
        Cost = {
            RessourcesNeeded = {
                { Resource = "STONE", Quantity = 1 },
		    }
	    },	
      })     
    else
        _root = ""
        _bottom = ""
      mod:registerAssetId("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. "_Part",     "PREFAB_" .. _nodePrefixUpper)
      mod:register({
      	DataType = "BUILDING_PART",
      	Id = _nodePrefixUpper .. "_PART",
      	Name = _nodePrefixUpper .. "_PART_NAME",
        Description = _nodePrefixUpper .. "_PART_DESC",
        ConstructorData = {
      		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
      		CoreObjectPrefab = "PREFAB_" .. _nodePrefixUpper
      	},
        Cost = {
            RessourcesNeeded = {
                { Resource = "WOOD", Quantity = 1 },
                { Resource = "STONE", Quantity = 1 },
		    }
	    },
      })
    end
    
    if _capacity > 0 then 
      mod:override({
            Id = _nodePrefixUpper .. "_PART",
            Capacity = _capacity, 
            AssetBuildingFunction = "BUILDING_FUNCTION_PUBLIC_LOUNGE_ROOM",
      })
    end
    
    -- change of tower scaling behaviour
    if _tableMenu == "tower" and _isScalable then
       mod:override({
          Id = _nodePrefixUpper .. "_PART",
          ConstructorData = {
              DataType = "BUILDING_CONSTRUCTOR_SCALER",
              CoreObjectPrefab = "PREFAB_" .. _nodePrefixUpper .. "_ROOT",
              ConstructionVisual = "PREFAB_" .. _nodePrefixUpper .. "_BOTTOM_CONSTR",
              EndPart = _nodePrefixUpper .. "_TOP",
              FillerList = {
                  _nodePrefixUpper .. "_MIDDLE",
              },
              MinimumScale = _minScale,
              IsVertical = true
          },
      }) 
    end     
    
    if      _tableMenu == "core" then table.insert(core, _nodePrefixUpper .. "_PART") 
    elseif  _tableMenu == "apse" then table.insert(apse, _nodePrefixUpper .. "_PART")
    elseif  _tableMenu == "narthex" then table.insert(narthex, _nodePrefixUpper .. "_PART")
    elseif  _tableMenu == "tower" then table.insert(tower, _nodePrefixUpper .. "_PART")
    elseif  _tableMenu == "roof" then table.insert(roof, _nodePrefixUpper .. "_PART")
    end
    
    if _isPlatform then 
        mod:configurePrefabFlagList("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. _bottom .. "_Part", { "PLATFORM" })  
        end

    -- Node registration-
    if _nodeType then
        mod:registerPrefabComponent("models/" .. _fbxPath .. ".fbx/Prefab/" .. _nodePrefixLower .. _root .. "_Part", {
            	DataType = "COMP_BUILDING_PART",
            	BuildingPartType = _nodeType
            })
    end
end

---------------------
-- Array of buildings
---------------------
--      Model,                          Scalable,   Fbx,    MinScale,   MenuList,   Capacity,   Platform,   Polygon    Type
local buildingPrefixList = {
	{  "Cathedral_Nave",                true,       "nave",     1,      "core",      20,         true,      {5,9},     false    },    
    {  "Cathedral_Aisle",               true,       "nave",     1,      "core",      10,         true,      {5,5},     false    },    
    {  "Cathedral_Crossing",            true,       "crossing", 1,      "core",      20,         true,      {9,9},     false    },    
    {  "Cathedral_Apse",                true,       "apse",     1,      "apse",      10,         true,      {9,9},     false    },    
    {  "Cathedral_Ambulatory",          true,       "apse",     1,      "apse",      0,          true,      {5,5},     false    },    
    {  "Cathedral_Radiating",           true,       "apse",     1,      "apse",      0,          true,      {5,8},     false    },    
    {  "Cathedral_Chapel",              true,       "apse",     1,      "apse",      0,          true,      {5,8},     false    },    
    {  "Cathedral_Narthex",             true,       "narthex",  1,      "narthex",   0,          true,      {7,9},     false    },    
    {  "Cathedral_Narthex_Tower_Medium",true,       "narthex",  1,      "narthex",   0,          true,      {7,7},     false    },    
    {  "Cathedral_Portal_Large",        true,       "portal",   0,      "narthex",   0,          true,      {4,2},     "DOOR"    },    
    {  "Cathedral_Portal_Medium",       true,       "portal_m", 0,      "narthex",   0,          true,      {3,2},     "DOOR"    },    
    {  "Cathedral_Filling_Large_Plain", true,       "narthex",  0,      "narthex",   0,          false,     {0,0},     false    },    
    {  "Cathedral_Window_Rose",         false,      "narthex",  0,      "narthex",   0,          false,     {0,0},     "RIGHT"    },    
    {  "Cathedral_ButtressFlying",      true,       "nave",     0,      "core",      0,          true,      {2,4},     false    },    
    {  "Cathedral_ButtressSolid",       true,       "nave",     0,      "core",      0,          true,      {2,4},     false    },    

    {  "Cathedral_Stairs_Large",        false,      "stairs",   0,      "narthex",   0,          true,      {2,4},     false    },    

    {  "Cathedral_Roof",                false,      "nave",     0,      "roof",      0,          false,     {0,0},     false    },    
    {  "Cathedral_Roof_Narthex",        false,      "nave",     0,      "roof",      0,          false,     {0,0},     false    },    
    {  "Cathedral_Roof_Front",          false,      "nave",     0,      "roof",      0,          false,     {0,0},     "LEFT"   },    
    {  "Cathedral_Roof_Apse",           false,      "nave",     0,      "roof",      0,          false,     {0,0},     false    },    
    {  "Cathedral_Roof_Aisle",          false,      "nave",     0,      "roof",      0,          false,     {0,0},     false    },    
    {  "Cathedral_Roof_Aisle_Corner",   false,      "nave",     0,      "roof",      0,          false,     {0,0},     false    },    
    {  "Cathedral_Roof_Ambulatory",     false,      "nave",     0,      "roof",      0,          false,     {0,0},     false    },    
    {  "Cathedral_Roof_Ambulatory_Outer", false,    "nave",     0,      "roof",      0,          false,     {0,0},     false    },    

    {  "Cathedral_Window_Apse_1",       true,       "apse",     0,      "apse",      0,          false,     {0,0},     "RIGHT"    },    
    {  "Cathedral_Window_Ambul_1",      true,       "apse",     0,      "apse",      0,          false,     {0,0},     "RIGHT"    },    
    {  "Cathedral_Window_Nave_1",       true,       "apse",     0,      "core",      0,          false,     {0,0},     "RIGHT"    },    
    {  "Cathedral_Window_Moulding",     false,      "apse",     0,      "core",      0,          false,     {0,0},     "RIGHT"    },    
    {  "Cathedral_Narthex_Moulding_1",  true,       "apse",     1,      "narthex",   0,          false,     {0,0},     "RIGHT"    },    
    
    {  "Tower_Small_Plain",             true,       "towers",   0,      "tower",     0,          false,     {0,0},     false    },    
    {  "Tower_Small_Narrow",            true,       "towers",   1,      "tower",     0,          false,     {0,0},     false    },    
    {  "Tower_Small_Wide",              true,       "towers",   1,      "tower",     0,          false,     {0,0},     false    },    
    {  "Tower_Small_Gallery",           false,      "towers",   0,      "tower",     0,          false,     {0,0},     false    },    
  
    {  "Tower_Medium_Plain",            true,       "towers",   0,      "tower",     0,          false,     {0,0},     false    },    
    {  "Tower_Medium_Narrow",           true,       "towers",   1,      "tower",     0,          false,     {0,0},     false    },    
    {  "Tower_Medium_Wide",             true,       "towers",   1,      "tower",     0,          false,     {0,0},     false    },    
    {  "Tower_Medium_Gallery",          false,      "towers",   0,      "tower",     0,          false,     {0,0},     false    },    
  
    {  "Tower_Window_Narrow_A",         true,       "towers",   0,      "tower",     0,          false,     {0,0},     "LEFT"    },    
    {  "Tower_Window_Narrow_B",         true,       "towers",   0,      "tower",     0,          false,     {0,0},     "LEFT"    },    
    {  "Tower_Window_Narrow_C",         true,       "towers",   0,      "tower",     0,          false,     {0,0},     "LEFT"    },    
    {  "Tower_Window_Wide_A",           true,       "towers",   0,      "tower",     0,          false,     {0,0},     "LEFT"    },    
    
    {  "Tower_Niche",                   false,      "towers",   0,      "tower",     0,          false,     {0,0},     "MINOR"    },    
    
}

---------------------
-- Registration
---------------------
for i, nodePrefixUpper in ipairs(buildingPrefixList) do
	registerBuildingPart(nodePrefixUpper)
end


---------------------
-- Node registration-
---------------------
mod:registerPrefabComponent("models/narthex.fbx/Prefab/Cathedral_Filling_Large_Plain__Root_Part", {
        	DataType = "COMP_BUILDING_PART",
        	BuildingPartType = "LEFT"
        })
mod:registerPrefabComponent("models/nave.fbx/Prefab/Cathedral_ButtressFlying__Root_Part", {
        	DataType = "COMP_BUILDING_PART",
        	BuildingPartType = "MINOR"
        })
mod:registerPrefabComponent("models/nave.fbx/Prefab/Cathedral_ButtressSolid__Root_Part", {
        	DataType = "COMP_BUILDING_PART",
        	BuildingPartType = "MINOR"
        })


-----------------------
-- Monument Building --
-----------------------
mod:register({
    DataType = "BUILDING",
    Id = "CATHEDRAL",
    Name = "CATHEDRAL_NAME",
    Description = "CATHEDRAL_DESC",
    BuildingType = "SERVICE",
    AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE", 
    AssetBuildingFunction = "BUILDING_FUNCTION_CHURCH",
    BuildingPartSetList = {
        {
            Name = "CATHEDRAL_LIST_CORE_NAME",
            BuildingPartList = core,
        },
        {
            Name = "CATHEDRAL_LIST_APSE_NAME",
            BuildingPartList = apse,
        },
        {
            Name = "CATHEDRAL_LIST_NARTHEX_NAME",
            BuildingPartList = narthex,
        },
        {
            Name = "CATHEDRAL_LIST_TOWER_NAME",
            BuildingPartList = tower,
        },
        {
            Name = "CATHEDRAL_LIST_ROOF_NAME",
            BuildingPartList = roof,
        },
    
    }
})






